typedef	void	ref_request_t;

/*
 *	This interface designed on the base of OpenGL library analisys.
 */

typedef enum
{
	ref_min_state	= 0,
	/* State part 1: managing viewports and world to screen transform */
	ref_viewport_width = 1,	/* state, array of one u32_t, ro */
	ref_viewport_height = 2,/* state, array of one u32_t, ro */
	ref_wts_overlay,	/* state, wo, NULL */
	ref_wts_ortho,		/* state, wo, array of 6 floats */
	ref_wts_frustum,	/* state, wo, array of 6 floats */
	/* State part 2: managing texturing mode */
	ref_texture,			/* boolean, wo */
	ref_texture_handle,		/* state, array of one u32_t, wo */
	/* NOTE: multitexturing may not be supported */
	ref_texture2,			/* boolean, wo */
	ref_texture2_handle,
	ref_texture_filter,		/* state, array of one ref_enum_t, wo */
	ref_nearest,			/* arg for ref_texture_filter */
	ref_linear,				/* arg for ref_texture_filter */
	ref_nearest_mipmap,		/* arg for ref_texture_filter */
	ref_linear_mipmap,		/* arg for ref_texture_filter */
	ref_texture_repeat,		/* boolean, wo */
	ref_texture_modulate,	/* boolean, wo */
	/* State part 3: managing blenging mode */
	ref_blend,		/* boolean, ro */
	ref_blend_add, 	/* state, wo, NULL */
	ref_blend_mult,	/* state; GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA */
	ref_alpha_test,	/* boolean, ro */
	ref_alpha_reference, /* state, array of one float */
	ref_max_state 	= 0xFFFF;
}	ref_enum_t;

/*
 *	TexHash can be 256 in len
 */
 
 
typedef struct refresh_device_s
{
	libv_object_t base;
	/* State control ------------------------------------------ */
	vboolean	(V_CALL	*set_state)(struct refresh_device_s *self,
		ref_enum_t pname,		/* Name of state variable */
		void *parm_vector);	/* Value of state variable */
	vboolean	(V_CALL	*get_state)(struct refresh_device_s *self,
		ref_enum_t pname,		/* Name of state variable */
		void *parm_vector);	/* Value of state variable */
	void		(V_CALL	*enable)(struct refresh_device_s *self,ref_enum_t pname);
	void		(V_CALL	*disable)(struct refresh_device_s *self,ref_enum_t pname);
	/* Modelview transform managing */
	/* NOTE: world-to-screen transformation must be set via 'set_state()';
	 * calls listed below intended for modelview transformations only */
	vboolean	(V_CALL	*push_transform)(struct refresh_device_s *self);
	vboolean	(V_CALL	*pop_transform)(struct refresh_device_s *self);
	vboolean	(V_CALL	*load_identity)(struct refresh_device_s *self);
	vboolean	(V_CALL	*load_transform)(struct refresh_device_s *self,
		matrix3x4_t transform);
	vboolean	(V_CALL	*mult_transform)(struct refresh_device_s *self,
		matrix3x4_t transform);
	vboolean	(V_CALL	*translate)(struct refresh_device_s *self,vec3_t offset);
	vboolean	(V_CALL	*scale)(struct refresh_device_s *self,vec3_t scales);
	vboolean	(V_CALL	*rotate)(struct refresh_device_s *self,vec3_t axis,vec_t radians);
	/* Framebuffer and texture management */
	void_p		(V_CALL	*find_texture_resource)(struct refresh_device_s *self,char *name);
	/* if 'alias == NULL', texture will not be cached */
	void_p		(V_CALL	*set_texture_data)(struct refresh_device_s *self,char *alias,
		u8_t *rgba_texels,u32_t width,u32_t height);
	vboolean	(V_CALL *overlay)(struct refresh_device_s *self,
		void_p texture_handle,
		u32_t src_x,u32_t src_y,
		u32_t src_width,u32_t src_height,
		u32_t dst_x,u32_t dst_y,
		u32_t dst_width,u32_t dst_height);
	vboolean	(V_CALL	*read_rgba_buffer)(struct refresh_device_s *self,
		u8_t **rgba_fb_contents,
		u32_t *fb_width,
		u32_t *fb_height);
	/* Graphics primitives */
	vboolean	(V_CALL	*begin_scene)(struct refresh_device_s *self);
	vboolean	(V_CALL	*clear)(struct refresh_device_s *self); /* all available buffers */
	vboolean	(V_CALL	*point)(struct refresh_device_s *self,ref_vertex_t *vert);
	vboolean	(V_CALL	*line)(struct refresh_device_s *self,ref_vertex_t *vert1,ref_vertex_t *vert2);
	vboolean	(V_CALL	*triangle)(struct refresh_device_s *self,ref_vertex_t *vert1,ref_vertex_t *vert2);
	vboolean	(V_CALL	*end_scene)(struct refresh_device_s *self);
}

#if 0

/*
 *	Texture object
 */
typedef struct ref_texture_s
{
	char	name[_MAX_PATH];
	void_p	handle;
}	ref_texture_t, *ref_texture_p;

/*
 *	Texture manager
 */
typedef struct ref_texture_manager_s
{
	bstack_t	texture_cache;
	u32_t		(V_CALL *find_texture)(struct ref_texture_manager_s *self,char *name);
	void_p		(V_CALL *get_texture_struct)(struct ref_texture_manager_s *self,u32_t index);
	void		(V_CALL *reload_textures)(void);
}

#endif

#if 0

typedef vboolean (V_CALL *ref_state_proc_t)(struct refresh_device_s *self,ref_enum_t pname,void *parm_vector);

typedef struct ref_base_s
{
	refresh_device_t vtbl;
	ref_state_proc_t	get_state_handlers[ref_max_state];
	ref_state_proc_t 	set_state_handlers[ref_max_state];
	ref_enable_proc_t	enable_handlers[ref_max_state];
	ref_disable_proc_t	disable_handlers[ref_max_state];
} ref_base_t, *ref_base_p;
	
#include <../source/argsused.h>
V_LOCAL vboolean V_CALL
ref_set_state_stub(struct refresh_device_s *self,ref_enum_t pname,void *parm_vector)
{
#ifdef	CHECKED_BUILD
	conlPrintf(LOG_WARNING,
		"ref->set_state(%u): unknown state; couldn't change.\n",pname);
#endif	/* CHECKED_BUILD */
	return false;
}

#include <../source/argsused.h>
V_LOCAL vboolean V_CALL
ref_get_state_stub(struct refresh_device_s *self,ref_enum_t pname,void *parm_vector);
{
#ifdef	CHECKED_BUILD
	conlPrintf(LOG_WARNING,
		"ref->get_state(%u): unknown state; couldn't retrieve.\n",pname);
#endif	/* CHECKED_BUILD */
	return false;
}

#include <../source/argsused.h>
V_LOCAL vboolean V_CALL
ref_enable_stub(struct refresh_device_s *self,ref_enum_t pname)
{
#ifdef	CHECKED_BUILD
	conlPrintf(LOG_WARNING,
		"ref->enable(%u): unknown pname; ignored.\n",pname);
#endif	/* CHECKED_BUILD */
	return false;
}

#include <../source/argsused.h>
V_LOCAL vboolean V_CALL
ref_disable_stub(struct refresh_device_s *self,ref_enum_t pname)
{
#ifdef	CHECKED_BUILD
	conlPrintf(LOG_WARNING,
		"ref->enable(%u): unknown pname; ignored.\n",pname);
#endif	/* CHECKED_BUILD */
	return false;
}

refAllocDriver(u32_t size_of_driver)
{
	ref_base_t *self;
	int i,num_state_handlers=(int)ref_max_state;
	if(size_of_driver<sizeof(ref_base_t))
	{
		/* Wrong descendant size */
		return NULL;
	}
	/* Allocate object */
	ALLOC_OBJECT(self,ref_base_t,size_of_driver-sizeof(ref_base_t),REF_MAGIC);
	/* Set all handlers to stubs */
	for(i=0;i<num_state_handlers;i++)
	{
		self->get_state_handlers[i]=ref_get_state_stub;
		self->set_state_handlers[i]=ref_set_state_stub;
		self->enable_handlers[i]=ref_enable_stub;
		self->disable_handlers[i]=ref_disable_stub;
	}
	/* Set other methods to stubs too */
	
}

V_LOCAL vboolean V_CALL
ref_base_get_state(struct refresh_device_s *self,
	ref_enum_t pname,void *parm_vector)
{
	/* All checks already performed in refSetState() */
	ref_base_t *dev=(ref_base_t*)self;
	return dev->get_state_handlers[pname](self,pname,parm_vector);
}

V_LOCAL vboolean V_CALL
ref_base_set_state(struct refresh_device_s *self,
	ref_enum_t pname,void *parm_vector)
{
	/* All checks already performed in refSetState() */
	ref_base_t *dev=(ref_base_t*)self;
	return dev->set_state_handlers[pname](self,pname,parm_vector);
}


refSetState()
{
	if((u32_t)pname < (u32_t)ref_max_state)
	{
	}
}

#endif

