/*	=======================================================================
**	Helper API.
*/

/*	=======================================================================
**	There are 8 ways to traverse:
**		- flag1: process first all leafs, than root;
**		- flag2: process from leftmost to rightmost sibling;
**		- flag3: process from root's leafs.
*/

void	V_CALL /* this is default, so it should be non-local */
traverse_lf_lf_uf(tree_traverse_proc_t self,ntree_p node,void *node_data,
	void (V_CALL *handler)(void*))
{
}

V_LOCAL	void	V_CALL
traverse_rf_lf_uf(tree_traverse_proc_t self,ntree_p node,void *node_data,
	void (V_CALL *handler)(void*))
{
}

V_LOCAL	void	V_CALL
traverse_lf_rf_uf(tree_traverse_proc_t self,ntree_p node,void *node_data)
{
}

V_LOCAL	void	V_CALL
traverse_rf_rf_uf(tree_traverse_proc_t self,ntree_p node,void *node_data)
{
}

V_LOCAL	void	V_CALL
traverse_lf_lf_df(tree_traverse_proc_t self,ntree_p node,void *node_data)
{
}

V_LOCAL	void	V_CALL
traverse_rf_lf_df(tree_traverse_proc_t self,ntree_p node,void *node_data)
{
}

V_LOCAL	void	V_CALL
traverse_lf_rf_df(tree_traverse_proc_t self,ntree_p node,void *node_data)
{
}

V_LOCAL	void	V_CALL
traverse_rf_rf_df(tree_traverse_proc_t self,ntree_p node,void *node_data)
{
}

void	V_LOCAL
treeTraverseFromRight_r(ntree_t *root,void (V_CALL *proc)(void*))
{
	u32_t ind,count;
	ntree_p *ch;
	if(!root)
		return;
	proc(root->data);
	ch=(ntree_p*)vpsGetStorage(&root->children);
	count=vpsCount(&root->children);
	if(!count)
		return;
	for(ind=count-1;;ind--)
	{
		treeTraverseFromRight_r(ch[ind],proc);
		if(!ind)
			return;
	}
}

void	V_LOCAL
treeTraverseFromLeftDU_r(ntree_t *root,void (V_CALL *proc)(void*))
{
	u32_t ind,count;
	ntree_p *ch;
	if(!root)
		return;
	ch=(ntree_p*)vpsGetStorage(&root->children);
	count=vpsCount(&root->children);
	for(ind=0;ind<count;ind++)
		treeTraverseFromLeftDU_r(ch[ind],proc);
	proc(root->data);
}

void	V_LOCAL
treeTraverseFromRightDU_r(ntree_t *root,void (V_CALL *proc)(void*))
{
	u32_t ind,count;
	ntree_p *ch;
	if(!root)
		return;
	ch=(ntree_p*)vpsGetStorage(&root->children);
	count=vpsCount(&root->children);
	if(!count)
		return;
	for(ind=count-1;;ind--)
	{
		treeTraverseFromRightDU_r(ch[ind],proc);
		if(!ind)
			break;
	}
	proc(root->data);
}

HERE

void	V_LOCAL
treeTraverseFromLeftRF_r(ntree_t *root,void (V_CALL *proc)(void*))
{
	u32_t ind,count;
	ntree_p *ch;
	if(!root)
		return;
	ch=(ntree_p*)vpsGetStorage(&root->children);
	count=vpsCount(&root->children);
	for(ind=0;ind<count;ind++)
		proc(ch[ind]->data);
	for(ind=0;ind<count;ind++)
		treeTraverseFromLeftRF_r(ch[ind],proc);
}

void	V_LOCAL
treeTraverseFromRightRF_r(ntree_t *root,void (V_CALL *proc)(void*))
{
	u32_t ind,count;
	ntree_p *ch;
	if(!root)
		return;
	ch=(ntree_p*)vpsGetStorage(&root->children);
	count=vpsCount(&root->children);
	if(!count)
		return;
	for(ind=count-1;;ind--)
	{
		proc(ch[ind]->data);
		if(!ind)
			break;
	}
	for(ind=count-1;;ind--)
	{
		treeTraverseFromRightRF_r(ch[ind],proc);
		if(!ind)
			return;
	}
}

vboolean V_LOCAL
#ifdef	CHECKED_BUILD
ntree_check_traverse_args(char *caller,ntree_t *node,void (V_CALL *proc)(void*))
{
	if(!node)
	{
		conlPrintf(LOG_ERROR,"%s(): nothing to traverse.\n",
			caller);
		return false;
	}
	if(!proc)
	{
		conlPrintf(LOG_ERROR,"%s(): nothing to do with nodes.\n",
			caller);
		return false;
	}
	return true;
}
#else	/* !CHECKED_BUILD */
ntree_check_traverse_args_q(btree_t *node,void (V_CALL *proc)(void*))
{
	return (!node || !proc) ? 0 : 1;
}
#define	ntree_check_traverse_args(caller,node,proc)	\
		ntree_check_traverse_args_q(node,proc)
#endif


/*
**	List of traverse procs
*/
V_LOCAL	tree_traverse_proc_t	traverse_procs[]=
{
	traverse_lf_lf_uf,
	traverse_rf_lf_uf,
	traverse_lf_rf_uf,
	traverse_rf_rf_uf,
	traverse_lf_lf_df,
	traverse_rf_lf_df,
	traverse_lf_rf_df,
	traverse_rf_rf_df
};

vboolean	V_GLOBAL
treeTraverseMode(tree_traverse_t pname,vboolean param)
{
	thread_data_t *td;
	u32_t bitmask,bits,shift;
	switch(pname)
	{
		case TREE_FROM_LEAFS:	shift=0; break;
		case TREE_FROM_LEFT:	shift=1; break;
		case TREE_FROM_UP:		shift=2; break;
		default:
#ifdef		CHECKED_BUILD
			conlPrintf(LOG_ERROR,
				"treeTraverseControl(%d,%d): unknown pname '%d'.\n",
				(int)pname,(int)param,(int)pname);
#endif		/* CHECKED_BUILD */
			return false;
	}
	bitmask=1 << shift;
	td=GetThreadData();
	bits=td->tree_traverse_mode;
	if(param)
		bits|=bitmask;
	else
		bits&=~bitmask;
	td->tree_traverse_mode=bits;
	td->tree_traverse_proc=traverse_procs[bits];
	return true;
}

typedef	void (V_CALL *tree_traverse_proc)(tree_traverse_proc*,ntree_p,void (V_CALL *handler)(void));

in thread_data_t:
	tree_traverse_proc_t traverse_proc;
*/

