/*<html><pre>*/
#ifndef V_LIB_H
#define V_LIB_H

/*
 *	======================================================================
 *                         V library master header file
 *	======================================================================
 *
 *	Purpose:	Another one library that - or only should be ! - platform-
 *				independent. Currently it can be compiled both under NT
 *				(and under MustDie - if you prefer it) and Linux.
 *				Linux seems to be VERY GOOD; but NT is the best ...
 *
 *	Version:	5.1 - working beta.
 *
 *	Language:	ANSI C. We can write object-oriented code in C, arn't we ?
 *
 *	Author:		Vladimir A. Smirnov aka ... hmmm ... not here, Ok ?
 *
 *	Type of routines:	Vector algebra, OpenGL connectivity; abstaction
 *						layer for IO (one set of function that
 *						performs reading both from disk and concatenated
 *						volumes); image loading (and JPEGLIB connectivity);
 *						logging and console input; general-purpose
 *						algorithms (stacks - growable arrays, fifo's);
 *						routines for vector graphics.
 *
 *	Project started:	Autumn, 1998.
 *	Last update: 		09.11.2001
 *
 *	Copyright:			Copyrighted under GNU GPL.
 *
 *	Bugs:		Under Win32, lot of broken - compiler related issues: for
 *				WATCOM - broken resource compiler; for Borland - strange
 *				linker, etc. I recommend to use MSVC.
 *	======================================================================
 */

/*
 *	All configuration information stored in another header.
 */
#include "v_conf.h"

/*
 * Main set of system headers: common for both platforms.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>

/*
 * 'math.h' from VisialC 5.0 is ANSI- and C++ - incompliant
 */
#ifdef		_MSC_VER
#pragma 	warning(disable:4001)	/* C++ - style comment */
#pragma 	warning(disable:4146)	/* unary minus applied to unsigned */
#pragma 	warning(disable:4514)	/* unreferenced inline func removed */
#endif		/* _MSC_VER */

#include	<math.h>

#ifdef		_MSC_VER
#pragma 	warning(default:4001)
#endif		/* _MSC_VER */

#ifndef		_MAX_PATH
#ifndef		MAX_PATH
#define		_MAX_PATH		256
#else		/* MAX_PATH */
#define		_MAX_PATH		MAX_PATH
#endif		/* MAX_PATH */
#endif		/* _MAX_PATH */

#ifndef		_MAX_DIR
#ifndef		MAX_DIR
#define		_MAX_DIR		256
#else		/* MAX_DIR */
#define		_MAX_DIR		MAX_DIR
#endif		/* MAX_DIR */
#endif		/* _MAX_DIR */

#ifndef		_MAX_FNAME
#ifndef		MAX_FNAME
#define		_MAX_FNAME		256
#else		/* MAX_FNAME */
#define		_MAX_FNAME		MAX_FNAME
#endif		/* MAX_FNAME */
#endif		/* _MAX_FNAME */

#ifndef		_MAX_EXT
#ifndef		MAX_EXT
#define		_MAX_EXT		256
#else		/* MAX_EXT */
#define		_MAX_EXT		MAX_EXT
#endif		/* MAX_EXT */
#endif		/* _MAX_EXT */

#ifndef		RIGHT_SLASHED
#ifndef		_MAX_DRIVE
#ifndef		MAX_DRIVE
#define		_MAX_DRIVE		4
#else		/* MAX_DRIVE */
#define		_MAX_DRIVE		MAX_DRIVE
#endif		/* MAX_DRIVE */
#endif		/* _MAX_DRIVE */
#endif		/* RIGHT_SLASHED */

#ifdef		CHECKED_BUILD
#include	<assert.h>
#define		V_ASSERT(exp)		assert(exp)
#else		/* CHECKED_BUILD */
#define		V_ASSERT(exp)
#endif		/* CHECKED_BUILD */

/*
 *	Default calling convention is 'cdecl' for UNIX/GNUC, 'stdcall' otherwise.
 */
#if			defined(__WIN32__) && !defined(__GNUC__)
#define		V_CALL		__stdcall
#else		/* __GNUC__ || !__WIN32__ */
#define		V_CALL
#endif		/* __GNUC__ */

/*
 *	Structure packing macro for GNUC
 */
#ifndef	ATTRIBUTE_PACKED
#ifndef	__GNUC__
#define	ATTRIBUTE_PACKED
#else	/* __GNUC__ */
#define	ATTRIBUTE_PACKED __attribute__((packed))
#endif	/* __GNUC__ */
#endif	/* !ATTRIBUTE_PACKED */

/*
 *	Macro prefix to export functions.
 *	NOTE: dynamic linking currently supported only under Win32.
 */
#ifdef		BUILD_DLL
#define		EXPORT				__declspec(dllexport)
#else		/* BUILD_DLL */
#define		EXPORT
#endif		/* BUILD_DLL */

#define		V_LOCAL			static
#define		V_GLOBAL		EXPORT	V_CALL

/*
 *	CBuilder refuses to import function with variable set of
 *	parameters if such declared as 'stdcall'.
 */
#if			defined(BCB_COMPAT) || defined(__BORLANDC__)
#define		V_GLOBAL_VA		EXPORT	__cdecl
#else		/* !BCB_COMPAT */
#define		V_GLOBAL_VA		V_GLOBAL
#endif		/* BCB_COMPAT */

/*
 *	Base integer, FP scalar and vector types.
 */
#include	"v_types.h"

#ifdef	__cplusplus
extern "C" {
#endif	/* __cplusplus */
	/*
	 *	These memory management functions used in all LibV routines.
     *	Some routines return memAlloc()'ed buffers which than must be
     *	released by memFree() call.
	 *	Implemented in TAL/objbase.c.
	 */
	void_p	V_GLOBAL	memAlloc(size_t size);
	void_p	V_GLOBAL	memRealloc(void *ptr,size_t newsize);
	void	V_GLOBAL	memFree(void *ptr);
	/*	------------------------------------------------------------------
	 *	Extra debugging stuff.
	 */
	/*
	 *	Returns memory information. Parameters:
	 *		- bytes_ret - pointer to store amount of allocated memory;
	 *		- blocks_ret - pointer to store number of allocated blocks;
	 *		- handles_ret - pointer to store number of created objects.
	 *	Note: bytes_ret is always 0 if compiled without MALLOC_DEBUG
	 */
	void		V_GLOBAL	memStats(u32_t *bytes_ret,u32_t *blocks_ret,
		u32_t *handles_ret);
    /* Prints memory info (using LibV console output) */
	void		V_GLOBAL	memPrintStats(void);
#ifdef	MALLOC_DEBUG
	/*
	 *	Sets new value for thread-specifig memory tag
	 */
	void	V_GLOBAL	memSetOpaque(unsigned opaque);
    /*
     *	Retuns 'true' if 'ptr' refers to memory blocks allocated
     *	by memAlloc().
     */
	vboolean	V_GLOBAL	memIsPointer(void *ptr);
    /* For each allocated object, prints detailed info */
	void		V_GLOBAL	memPrintObjStats(void);
    /* For each allocated block, prints detailed info */
	void		V_GLOBAL	memPrintMemStats(void);
#endif	/* MALLOC_DEBUG */
#ifdef	__cplusplus
}
#endif	/* __cplusplus */

/*
 *		Workaround for MS cl.exe + tlink32. Is this still needed ?
 */
#if		defined(BUILD_DLL) && defined(BCB_COMPAT) && !defined(__BORLANDC__)
#define	conPrintf	_conPrintf
#define	conlPrintf	_conlPrintf
#define	filePrintf	_filePrintf
#define	vglPrintf	_vglPrintf
#endif

/*
 *	Other LibV headers.
 *	TODO: rename them.
 */
#include	"v_x86.h"
#include	"v_math.h"
#include	"v_vector.h"
#include	"v_gpsp.h"
#include	"v_log.h"
#include	"v_matrix.h"
#include	"v_plane.h"
#include	"v_space.h"
#include	"v_time.h"
#include	"v_fifo.h"
#include	"v_parse.h"
#include	"v_objlib.h"
#include	"v_pgrid.h"
#include	"v_pprof.h"
#include	"v_fs.h"
#include	"v_image.h"
#include	"v_str.h"
#include	"v_stack.h"
#include	"v_tree.h"
#include	"v_comm.h"
#include	"v_input.h"
/*
 * 29.11.2001.
 */
#ifndef		__UNIX__
#include	"sgi/gl.h"
#else		/* __UNIX__ */
#include	"GL/gl.h"
#endif		/* __UNIX__ */

#include	"v_camctl.h"
#include	"v_gl.h"
#include	"v_wmf.h"
#include	"v_tlist.h"
#include	"v_fbdraw.h"
#include	"v_sched.h"
#include	"v_wavlib.h"
#include	"v_wavin.h"
#include	"v_wavout.h"
#include	"v_reg.h"

/*
 *	ANSI version of some functions HAS underscores; but UNIX libc and
 *	all Win32 RTLs (except MS Visual C RTL) uses functions without them.
 */
#ifndef		_MSC_VER
#define		_stricmp	strcmpi
#define		_chdir		chdir
#define 	_mkdir      mkdir
#define		_getcwd		getcwd
#endif		/* _MSC_VER */

/*
 *	GNU / UNIX libc has strncasecmp(); MS / BORLAND RTL has strnicmp.
 *	For WATCOM C we implement strncasecmp() in TAL/string.c.
 */
#if			!defined(__UNIX__) && !defined(__GNUC__) && \
			!defined(__WATCOMC__)
#define		strcasecmp		_stricmp
#define		strncasecmp		_strnicmp
#endif		/* !defined(__UNIX__) */

#endif		/* V_LIB_H */
/*</pre></html>*/
/*EOF*/
