/*<html><pre>*/
#ifndef TAL_OBJBASE_H
#define TAL_OBJBASE_H

/*
**	LibV internal master header: object manager.
*/ 

#ifdef	V_LIB_H
#error	This file must be #included prior to 'v_lib.h'.
#endif	/* V_LIB_H */

#define	LIBV_INTERNAL
#include <v_lib.h>
#include <../source/PAL/dataman.h>

/*
**	Macros used by memory management functions.
*/
#define	MEMORY_HASHPROC(ptr) \
		(((u32_t)(ptr) + ((u32_t)(ptr) >> 4) + \
          ((u32_t)(ptr) >> 8)) & MEMORY_HASH_LIMIT)

#define	OBJECT_HASHPROC(ptr) \
		(((u32_t)(ptr) + ((u32_t)(ptr) >> 4) + \
          ((u32_t)(ptr) >> 8)) & OBJECT_HASH_LIMIT)

/*
**	Base object for most LibV objects.
*/
typedef struct libv_object_s
{
#ifdef	CHECKED_BUILD
	u32_t	magic;
#endif	/* CHECKED_BUILD */
	u32_t	ref_count;
	u32_t	(V_CALL *destroy)(void *self);
	/*
	**	May be this looks as herecy, but probably this is not ...
	**	This seems to be a good way to make objects extensible.
	vboolean	(V_CALL *query_interface)(u32_t ref_id,
		object_handle_t *handle_ret);
	*/
}	libv_object_t, *libv_object_p;

/*
**	Macro to inject DWORD object identifier.
*/
#ifdef	CHECKED_BUILD
#define	MARK_OBJECT(object,magic)	*((u32_t*)(object))=(u32_t)(magic)
#else	/* !CHECKED_BUILD */
#define	MARK_OBJECT(object,magic)
#endif	/* !CHECKED_BUILD */

/*
**	Constructor.
**	Implemenation as macro is much more useful, also less intuitive.
*/
#define	ALLOC_OBJECT(ptr,type,extra_memory,magic) \
		{ \
			u32_t cksize = sizeof(type)+(extra_memory); \
			(ptr)=(type*)memAllocObject(cksize); \
            V_ASSERT(ptr); \
	    	if(!(ptr)) \
            	return NULL; \
			memZero((ptr),cksize); \
            ((libv_object_t*)(ptr))->ref_count=1; \
			((libv_object_t*)(ptr))->destroy=baseDestroy; \
			/* ((libv_object_t*)(ptr))->query_interface=baseQueryInterface; */ \
			MARK_OBJECT(ptr,magic); \
        }

#ifdef	__cplusplus
extern "C" {
#endif	/* __cplusplus */
	/*
    **	Object-specific memory management.
    */
	object_handle_t V_CALL	memAllocObject(size_t size);
	void		V_GLOBAL	memFreeObject(object_handle_t ptr);
    /*
     *	Returns 'true' if 'ptr' was allocated via call to
     *	memAllocObject().
     */
	vboolean		V_CALL	memIsObject(object_handle_t ptr);
	/* Garbage collector to call from threadJoin(). */
	void	V_CALL	libvCloseHandlesAndFreeMemory(thread_data_t
    	*thread_data);
	/*
	**	Function for testing object handles.
	**	Returns NULL (and complains an error) if:
	**		a)	agrument was not created via memAllocObject();
	**		b)	(when CHECKED_BUILD) first DWORD of memory pointed
	**			by 'handle' != magic.
	**	Otherwise returns 'handle' back.
	**	Implemented in TAL/objbase.c.
	*/
#ifdef	CHECKED_BUILD
	void_p	V_CALL	libvIsHandle(object_handle_t handle,
		char *caller,	/* Name of function handle passed to */
		char *object,	/* Sence of handle: file, ipc, etc... */
		u32_t magic);	/* Magic: must be in FIRST DWORD of object */
#else	/* !CHECKED_BUILD */
#define	libvIsHandle(handle,caller,object,magic)	(handle)
#endif	/* CHECKED_BUILD */
	/*
    **	Function for closing object handles.
    **	Decreases reference counter, if it reaches 0 - calls destructor.
    **	Returns new reference count:
    **		0 - object no longer exists;
    **		> 0 - there are still references to object.
    */
#ifdef	CHECKED_BUILD
	u32_t V_CALL	libvCloseHandle(object_handle_t handle,
		char *caller,	/* Name of function handle passed to */
		char *object,	/* Sence of handle: file, ipc, etc... */
		u32_t magic);	/* Magic: must be in FIRST DWORD of object */
#else	/* !CHECKED_BUILD */
#define	libvCloseHandle(handle,caller,object,magic)	\
		__libvCloseHandle(handle)
#endif	/* CHECKED_BUILD */
	/* Quiet version of libvCloseHandle() */
	u32_t	V_CALL	__libvCloseHandle(object_handle_t handle);
	/*	Default destructor for base object */
	u32_t	V_CALL	baseDestroy(object_handle_t handle);
#ifdef	__cplusplus
}
#endif	/* __cplusplus */

#endif	/* TAL_OBJBASE_H */

/*</pre></html>*/
/*EOF*/

