/*<html><pre>*/
#include <../source/TAL/objbase.h>

/* Fortran code comes form: Shikin, Plis. Curves and surfaces... */  
V_LOCAL	void
csplineSolveLE(mvec_t *a,mvec_t *b,mvec_t *c,mvec_t *d,mvec_t *u,
	mvec_t *v,mvec_t *w,mvec_t *s,mvec_t *t,mvec_t *x,u32_t n)
{
	u32_t i,i1;
	mvec_t z;
	u[0]=0;
	v[0]=0;
	w[0]=1;
	for(i=1;i<n;i++)
	{
		i1=i+1;
		z=(mvec_t)1.0/(a[i]+c[i]*v[i]);
		v[i1]=-b[i]*z;
		u[i1]=(-c[i]*u[i]+d[i])*z;
		w[i1]=-c[i]*w[i]*z;
	}
	s[n-1]=1;
	t[n-1]=0;
	for(i=n-2;;)
	{
    	if(!(i--))
    		break;
		i1=i+1;
    	s[i]=v[i1]*s[i1]+w[i1];
	    t[i]=v[i1]*t[i1]+u[i1];
    }
	x[n-1]=(d[n-1]-b[n-1]*t[0]-c[n-1]*t[n-2])/
		(a[n-1]+b[n-1]*s[0]+c[n-1]*s[n-2]);
	n--;
	for(i=0;i<n;i++)
	    x[i]=s[i]*x[n]+t[i];	/* n = n-1 at this time */
}

typedef	struct cspline_s
{
	libv_object_t base;
	u32_t 		n,ib;
	u32_t		ind;
	mvec_p		internal_x_storage;
	mvec_p		internal_y_storage;
	mvec_t		ax, bx;
	mvec_p	x;
	mvec_p	y;
	mvec_p	a;
	mvec_p	b;
	mvec_p	c;
	mvec_p	d;
	mvec_p	z;
	mvec_p	u;
	mvec_p	v;
	mvec_p	w;
	mvec_p	t;
	mvec_p	s;
}	cspline_t, *cspline_p;

#ifdef	CHECKED_BUILD
#define	CSP_MAGIC			0x4453504C	/* CSPL */
V_LOCAL	char_p	csp_object	= "cubic spline";
#endif	/* CHECKED_BUILD */

V_LOCAL	void
__cspInterpolate(cspline_p sp,mvec_t xx,mvec_t *res)
{
	u32_t j,ip;
	u32_t ne,ns,nf;
	mvec_t cc,h1,h2,am,al,g0,gn,tt;
	mvec_t ax=sp->ax;
	mvec_t bx=sp->bx;
	u32_t n=sp->n;
	u32_t ib=sp->ib;
	mvec_p x=sp->x;
	mvec_p y=sp->y;
	mvec_p a=sp->a;
	mvec_p b=sp->b;
	mvec_p c=sp->c;
	mvec_p d=sp->d;
	mvec_p z=sp->z;
	mvec_p u=sp->u;
	mvec_p v=sp->v;
	mvec_p w=sp->w;
	mvec_p t=sp->t;
	mvec_p s=sp->s;
	ip=0;
	g0=gn=0;
	if(sp->ind != 1)
	{
	    a[0]=2;
		ne=n;
    	ns=2;
    	nf=n-1;
    	switch(ib)
    	{
    		case 1:
				b[0]=0;
        		c[0]=0;
		        d[0]=2*ax;
        		a[n-1]=2;
		        b[n-1]=0;
        		c[n-1]=0;
		        d[n-1]=2*bx;
	      		break;
    		case 2:
				b[0]=1;
                c[0]=0;
                h1=x[1]-x[0];
                d[0]=3*(y[1]-y[0])/h1-(mvec_t)0.5*h1*ax;
                a[n-1]=2;
                b[n-1]=0;
                c[n-1]=1;
                h1=x[n-1]-x[n-2];
                d[n-1]=3*(y[n-1]-y[n-2])/h1+(mvec_t)0.5*h1*bx;
      			break;
      		case 3:
				h1=x[1]-x[0];
				h2=x[n-1]-x[n-2];
				am=h2/(h1+h2);
        		al=1-am;
		        b[0]=am;
		        c[0]=al;
        		d[0]=3*(am*(y[1]-y[0])/h1+al*(y[0]-y[n-2])/h2);
        		h1=x[n-2]-x[n-3];
        		h2=x[n-1]-x[n-2];
        		am=h1/(h1+h2);
		        al=1-am;
		        a[n-2]=2;
        		b[n-2]=am;
		        c[n-2]=al;
        		d[n-2]=3*(am*(y[n-1]-y[n-2])/h2+al*(y[n-2]-y[n-3])/h1);
        		nf=n-2;
        		ne=n-1;
      			break;
      		case 4:
                h1=x[1]-x[0];
                h2=x[2]-x[1];
                g0=h1/h2;
                a[1]=1+g0;
                b[1]=g0;
                c[1]=0;
                am=h1/(h1+h2);
                al=1-am;
                cc=am*(y[2]-y[1])/h2+al*(y[1]-y[0])/h1;
                d[1]=cc+2*g0*(y[2]-y[1])/h2;
                h2=x[n-1]-x[n-2];
                h1=x[n-2]-x[n-3];
                gn=h1/h2;
                a[n-2]=1+gn;
                b[n-2]=0;
                c[n-2]=gn;
                am=h1/(h1+h2);
                al=1-am;
                cc=am*(y[n-1]-y[n-2])/h2+al*(y[n-2]-y[n-3])/h1;
                d[n-2]=cc+2*gn*(y[n-2]-y[n-3])/h1;
                ns=3;
                nf=n-2;
                ne=nf;
                ip=1;
				break;
			default:	/* unknown boundary condition */
				return;
		}	/* switch(ib) */
		for(j=ns-1;j<nf;j++)
		{
			h1=x[j+1]-x[j];
			h2=x[j]-x[j-1];
			am=h2/(h1+h2);
			al=1-am;
			c[j]=al;
			a[j]=2;
			b[j]=am;
			d[j]=3*(am*(y[j+1]-y[j])/h1+al*(y[j]-y[j-1])/h2);
		}
		csplineSolveLE(a+ip,b+ip,c+ip,d+ip,u,v,w,s,t,z+ip,ne);
		if(ib==3)
			z[n-1]=z[0];
		else if(ib==4)
		{
			z[0]=g0*g0*z[2]+(g0*g0-1)*z[1]+2*((y[1]-y[0])/(x[1]-x[0])-
				g0*g0*(y[2]-y[1])/(x[2]-x[1]));
       		z[n-1]=gn*gn*z[n-3]+(gn*gn-1)*z[n-2]+
				2*((y[n-1]-y[n-2])/(x[n-1]-x[n-2])-
	          	gn*gn*(y[n-2]-y[n-3])/(x[n-2]-x[n-3]));
      	}
		sp->ind=1;
	} /* if(!ind) */
	/* TODO: convert to bsearch() ! */
  	for(j=1;j<n && x[j]<=xx;j++);
	j--;
	tt=(xx-x[j])/(x[j+1]-x[j]);
	*res=y[j]*(1-tt)*(1-tt)*(1+2*tt)+y[j+1]*tt*tt*(3-2*tt)+
		z[j]*(x[j+1]-x[j])*tt*(1-tt)*(1-tt)-z[j+1]*tt*tt*(1-tt);
}

V_LOCAL	cspline_p
cspAlloc(u32_t n)
{
	cspline_t *sp;
	size_t len_limit=n+2;
	size_t num_arrays=12; /* see definition os struct csp */
	size_t extra_size=num_arrays*len_limit*sizeof(mvec_t);
    ALLOC_OBJECT(sp,cspline_t,extra_size,CSP_MAGIC);
	sp->x=sp->internal_x_storage=(mvec_t*)(sp+1);
	sp->y=sp->internal_y_storage=sp->x+len_limit;
	sp->a=sp->y+len_limit;
	sp->b=sp->a+len_limit;
	sp->c=sp->b+len_limit;
	sp->d=sp->c+len_limit;
	sp->z=sp->d+len_limit;
	sp->u=sp->z+len_limit;
	sp->v=sp->u+len_limit;
	sp->w=sp->v+len_limit;
	sp->t=sp->w+len_limit;
	sp->s=sp->t+len_limit;
	sp->ib=2;
	sp->n=n;
	return sp;
}

object_handle_t V_GLOBAL
cspCreate(vector_t *vx,vector_t *vy)
{
	u32_t n;
	cspline_t *sp;
	if(!vx || !vy)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,
			"cspCreate(%X,%X): incorrect parameter(s).\n",vx,vy);
#endif	/* CHECKED_BUILD */
		return NULL;
	}
	n=vx->size;
	if(n!=vy->size)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,
			"cspCreate(): (%X)->size=%u, but (%X)->size=%u.\n",
			vx,vy,vx->size,vy->size);
#endif	/* CHECKED_BUILD */
		return NULL;
	}
	sp=cspAlloc(n);
    if(!sp)
    	return NULL;
	memCopy(sp->x,vx->elements,n*sizeof(mvec_t));
	memCopy(sp->y,vy->elements,n*sizeof(mvec_t));
	return (object_handle_t)sp;
}

mvec_t	V_GLOBAL
cspInterpolate(object_handle_t csp,mvec_t x)
{
	mvec_t y;
	if(!libvIsHandle(csp,"cspInterpolate",csp_object,CSP_MAGIC))
    	return 0;
	__cspInterpolate((cspline_p)csp,x,&y);
	return y;
}

void	V_GLOBAL
cspChangeX(object_handle_t csp,vector_t *new_x)
{
	cspline_t *sp;
	if(!libvIsHandle(csp,"cspChangeX",csp_object,CSP_MAGIC))
    	return;
    sp=(cspline_t*)csp;
	if(sp->n != new_x->size)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,"cspChangeX(%X,%X): expected size of vector "
			"'%d', found '%d'.\n",csp,new_x,sp->n,new_x->size);
#endif	/* CHECKED_BUILD */
		return;
	}
	memCopy(sp->x,new_x->elements,sp->n*sizeof(mvec_t));
	sp->ind=0;
}

void	V_GLOBAL
cspChangeY(object_handle_t csp,vector_t *new_y)
{
	cspline_t *sp;
	if(!libvIsHandle(csp,"cspChangeY",csp_object,CSP_MAGIC))
    	return;
    sp=(cspline_t*)csp;
	if(sp->n != new_y->size)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,"cspChangeY(%X,%X): expected size of vector "
			"'%d', found '%d'.\n",csp,new_y,sp->n,new_y->size);
#endif	/* CHECKED_BUILD */
		return;
	}
	memCopy(sp->y,new_y->elements,sp->n*sizeof(mvec_t));
	sp->ind=0;
}

void	V_GLOBAL
cspSetBoundaryCondition(object_handle_t csp,u32_t cond,mvec_t cond_left,
	mvec_t cond_right)
{
	cspline_t *sp;
	if(!libvIsHandle(csp,"cspSetBoundaryCondition",csp_object,CSP_MAGIC))
    	return;
    sp=(cspline_t*)csp;
	if(cond<1 || cond>4)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,"cspSetBoundaryCondition(%X,%u,%g,%g): wrong "
			"condition '%d'.\n",csp,cond,cond_left,cond_right,cond);
#endif	/* CHECKED_BUILD */
		return;
	}
	sp->ib=cond;
	sp->ax=cond_left;
	sp->bx=cond_right;
	sp->ind=0;
}

void	V_GLOBAL
cspDestroy(object_handle_t csp)
{
	libvCloseHandle(csp,"cspDestroy",csp_object,CSP_MAGIC);
}

/*</pre></html>*/
/*EOF*/
