#ifndef	V_STROKE_H
#define	V_STROKE_H

/*
 *		General notes about 'geometry' classes.
 *		There are several ways to define meshes. Most intuitive way is
 *	to define mesh as array of complex type 'vertex'; later must
 *	hold both position and extended information: color, texture coord, etc.
 *		The main disadventage of such definition is that extended
 *	information is optional and may not be used in several cases.
 *
 *	So, instead, we define several arrays for storing:
 *		a) geometry - meshStrokes object;
 *		b) extended vertex and face information - meshAttribs
 *		b) topology - meshTopology object;
 */

/*
 *		Geometry information in 'meshStrokes' arranged in three-level
 *	hierarcy: coords assebmled in 'mesh_stroke' (leter normally used
 *	for linestrip representation, but can be used for tris, tristrips and
 *	trifans, also); mesh_stroke's assembled in 'glyphs' (normally used
 *	for character representation, but can also be used for geometry
 *	objects); finally, 'glyphs' assembled in 'meshStrokes'.
 *		Ways for use of geometry info can vary, and, thus, there are
 *	no methods to render geometry in 'meshStrokes'.
 *		Extended information can be optionally associated with each
 *	geometry vertex. For that purpose there is secial placeholder in
 *	'mesh_stroke', namely 'attribs'.
 */
  
typedef struct	mesh_stroke_s	/* single polyline */
{
	u32_t		num_coords;
    u32_t		num_attribs;
	vec3_t		*coords;
	u32_t		*attribs;
}	mesh_stroke_t, *mesh_stroke_p;

typedef struct mesh_topology_s /* topology information */
{
	int			dummy;
}	mesh_topology_t, *mesh_topology_p;

typedef struct	vertex_attrib_s
{
	u32_t		color;
    u32_t		texcoord;
	u8_t		rgba[4];		/* color vertex */
}	vertex_attrib_t, *vertex_attrib_p;
    
#ifndef	GL_VERTEX3V
#ifdef	DOUBLEVEC
#define	GL_VERTEX3V	glVertex3dv
#else	/* !DOUBLEVEC */
#define	GL_VERTEX3V	glVertex3fv
#endif	/* !DOUBLEVEC */
#endif	/* !GL_VERTEX3V */

#define	DRAW_STROKE(stroke) { \
			u32_t i,len=(stroke).num_coords; \
            glBegin(GL_LINE_STRIP); \
			for(i=0;i<len;i++) \
				GL_VERTEX3V((stroke).coords[i]); \
			glEnd(); \
		}

#ifdef	__cplusplus
extern	"C" {
#endif		/* __cplusplus */
	/* Constructor and destructor */
	object_handle_t	V_GLOBAL	meshCreateStrokes(void);
    void	V_GLOBAL	meshDeleteStrokes(object_handle_t ms);
    /* Methods for creation of glyphs */
	void	V_GLOBAL	meshStrokeCoord(object_handle_t ms,
    	vec_t x,vec_t y,vec_t z);
	void	V_GLOBAL	meshStrokeCoordv(object_handle_t ms,vec3_t xyz);
	void	V_GLOBAL	meshEndStroke(object_handle_t ms);
	void	V_GLOBAL	meshCancelStroke(object_handle_t ms);
	void	V_GLOBAL	meshEndGlyph(object_handle_t ms);
	void	V_GLOBAL	meshCancelGlyph(object_handle_t ms);
	/* Returns number of complete glyphs */
	u32_t	V_GLOBAL	meshGlyphCount(object_handle_t ms);
	/* Returns number of complete strokes for uncomplete glyph */
	u32_t	V_GLOBAL	meshStrokesCount(object_handle_t ms,
    	u32_t glyph_num);
	/* Returns stroke # 'stroke_num' of glyph # 'glyph_num' */
    mesh_stroke_p	V_GLOBAL meshGlyphStroke(object_handle_t ms,
    	u32_t glyph_num,u32_t stroke_num);
#ifdef	__cplusplus
}
#endif		/* __cplusplus */

#endif	/* V_STROKE_H */
