TODO

/*<html><pre>*/
#ifndef	V_MESH_H
#define V_MESH_H

/*
**	Uses concepts of various programs.
**	Among them: Quake & 3DSMAX.
*/

/*
**	Static mesh (geomobject_t) represent single solid object.
*/

/*	Mesh vertex */
typedef struct mesh_vertex_s
{
	u32_t	vertex;				/* Index of geometry vertex */
	u32_t	texcoord;			/* Index of texture vertex / texinfo */
	u32_t	normal;				/* Index of normal vector */
	u32_t	color;				/* Index of vertex color */
}	mesh_vertex_t, *mesh_vertex_p;

/*	Triangular face to pass to renderer */
typedef	mesh_vertex_t	mesh_face_t[3];

/*	Interface to generic map */
typedef	struct mesh_map_s
{
	libv_object_t	base;
    vboolean	(V_CALL *reload)(object_handle_t self);
    void		(V_CALL *bind)(object_handle_t self);
} mesh_map_t, *mesh_map_p;

/*	Interface to generic material */
typedef struct	mesh_mtl_s
{
	libv_object_t	base;
    u32_t		(V_CALL *bind)(object_handle_t self);
    vboolean	(V_CALL *reload)(object_handle_t self);
}	mesh_mtl_t, *mesh_mtl_p;

/*	Solid object */
typedef struct mesh_object_s
{
	char	name[64];			/* Object name */
	/* Topology */
    
		int			numVerts;
		int	 		numFaces;
		Face *		faces;

		// Geometry
		Point3 *	verts;		

		// Texture Coord assignment 
		int			numTVerts;
		UVVert *	tVerts;
		TVFace *	tvFace;  	 
		Point3 *	t3DVerts;	// Snap shot of regular vertices before a deformation

		// Color per vertex
		int numCVerts;
		VertColor * vertCol;
		TVFace *    vcFace;

		// Material assignment
		MtlID		mtlIndex;     // object material
		MtlID	* 	faceMtlIndex; // material index per face. 

		// Selection
		BitArray	vertSel;  		// selected vertices
		BitArray	faceSel;  		// selected faces
		BitArray	edgeSel;		// selected edges, identified as 3*faceIndex + edgeIndex
		BitArray	vertHide;		// Hide flags for vertices

		// Display attribute flags
		DWORD		dispFlags;

		// Selection level
		DWORD		selLevel;

		// true if normals have been built for the current mesh
		int			normalsBuilt;

		MeshRenderData*	 renderData;  // used by the renderer

		// derived data-- can be regenerated
		StripTab		*stab;			// depends on topo


    u32_t	mtlid;				/* Material ID */
	char	material[64];		/* Material name */
	u32_t	mtlid;				/* Valid only after mtlLoad() */
	u32_t	startface;			/* Index of start face */
	u32_t	num_faces;			/* Number of faces */
	vec3_t	mins, maxs;			/* Object bounding box */
	vec3_t	pivot;				/* Pivot point */
	matrix3x4_t xform;			/* XForm to be applied when drawing */
	u32_t	faces_flipped;		/* Flip order of vertices when saving */
}	mesh_object_t, *mesh_object_p;

/*	Static set of solid objects */
typedef struct	mesh_s
{
	libv_object_t base;
	bstack_t	vertices;		/* Vertex pool */
    bstack_t	texcoords;		/* Texture coord. pool */
	bstack_t	normals;		/* Vert. normal vector pool */
	bstack_t	faces;			/* Face pool */
	bstack_t	face_centers;	/* Face centers for Z-sort */
	bstack_t	face_planes;	/* Face normals & dists */
	bstack_t	objects;		/* Object pool */
	bstack_t	colors;			/* Color vertices */
	bstack_t	maps;			/* Texture maps */
	bstack_t	mtls;			/* Materials */
}	mesh_t, *mesh_p;

#ifdef	__cplusplus
extern "C" {
#endif	/* __cplusplus */
	/* Texture managing: creates/destroys texture maps.
     * Implemented in geom/meshmap.c. */
	object_handle_t	V_GLOBAL	meshCreateTextureMap(char_p image_file);
	void 		V_GLOBAL	meshDestroyTextureMap(object_handle_t self);
	/* Mesh managind.
    
	object_handle_t	V_GLOBAL	meshCreate(void);
    u32_t			V_GLOBAL	meshDestroy(object_handle_t mesh);
    /*
     *	Note: meshClear() just sets all counters to 0, but NOT
     *	memFree()'s all blocks; it can be called every frame
     *	without significant performance loss.
     */
    void	V_GLOBAL	meshClear(object_handle_t mesh);
    /*
     *	Querring functions.
     */
    
    
		u32_t num_vertices,
		u32_t num_texcoords,
		u32_t num_normals,
		u32_t num_faces,
		u32_t num_objects,
		u32_t num_colors,
		u32_t num_maps,
		u32_t num_mtls);
	/* Returns malloc()'ed block */
	mesh_p		V_GLOBAL	objLoad(char *filename);
	/* Do NOT frees any memory, just saves mesh */
	vboolean	V_GLOBAL	objSave(char *filename,mesh_t *obj);
#ifdef	__cplusplus
}
#endif	/* __cplusplus */

#endif	/* V_OBJLIB_H */
/*</pre></strong></body></html>*/
/*EOF*/
