/*<html><pre>*/
#include <../source/TAL/objbase.h>

/*
 *	fftwrap.c
 *	This module contains both platform-specific wrappers
 *	and simple implementation for complex Fast Fourier Transform.
 */
#define	NO_GSL

#if		defined(__WIN32__) && !defined(NO_NSP)
#define	HAS_NSP
#elif	defined(__UNIX__) && !defined(NO_GSL)
#define	HAS_GSL
#endif	/* __UNIX__ */

#define	power2(op)	(1<<(op))

#ifdef	HAS_NSP

#include "../../support/nsp/include/nsp.h"
#include "../../support/nsp/include/nspfft.h"

void	V_GLOBAL
fftCalcNip(
	mvec_t *ReOut,mvec_t *ImOut,
	mvec_t *ReIn,mvec_t *ImIn,
	unsigned size_power,mvec_t dir_sign)
{
	if(!ReOut || !ImOut || !ReIn || !ImIn || !size_power ||
		IS_POWER_TWO(size_power))
	{
#ifdef	CHECKED_BUILD
		conlPuts(LOG_ERROR,"fftCalcNip(): incorrect parameters.\n");
#endif	/* CHECKED_BUILD */
		return;
	}
	if(!ReOut || !ImOut || !ReIn || !ImIn || !size_power)
	{
#ifdef	CHECKED_BUILD
		conlPuts(LOG_ERROR,"fftCalcNip(): incorrect parameters.\n");
#endif	/* CHECKED_BUILD */
		return;
	}
	if(dir_sign > 0)	/* Forward FFT */
	{
#ifdef	MATRIX_DOUBLE
		nspzrFftNip(ReIn,ImIn,ReOut,ImOut,size_power,NSP_Forw);
#else	/* !MATRIX_DOUBLE */
		nspcrFftNip(ReIn,ImIn,ReOut,ImOut,size_power,NSP_Forw);
#endif	/* !MATRIX_DOUBLE */
	}
	else				/* Inverse FFT */
	{
#ifdef	MATRIX_DOUBLE
		nspzrFftNip(ReIn,ImIn,ReOut,ImOut,size_power,NSP_Inv);
#else	/* !MATRIX_DOUBLE */
		nspcrFftNip(ReIn,ImIn,ReOut,ImOut,size_power,NSP_Inv);
#endif	/* !MATRIX_DOUBLE */
	}
}

#elif	defined(HAS_GSL) /* !HAVE_NSP */

#include <gsl_errno.h>
#include <gsl_fft_complex.h>

void	V_GLOBAL
fftCalcNip(
	mvec_t *ReOut,mvec_t *ImOut,
	mvec_t *ReIn,mvec_t *ImIn,
	unsigned size_power,mvec_t dir_sign)
{
	size_t i,len;
	double *packed;
	if(!ReOut || !ImOut || !ReIn || !ImIn || !size_power ||
		IS_POWER_TWO(size_power))
	{
#ifdef	CHECKED_BUILD
		conlPuts(LOG_ERROR,"fftCalcNip(): incorrect parameters.\n");
#endif	/* CHECKED_BUILD */
		return;
	}
	len=power2(size_power);
	packed=(double*)memAlloc((len << 1) * sizeof(double));
	for(i=0;i<len;i++)
	{
		packed[i<<1]=ReIn[i];
		packed[(i<<1)+1]=ImIn[i];
	}
	if(dir_sign > 0)	/* Forward FFT */
		gsl_fft_complex_radix2_forward(packed,1,len);
	else				/* Inverse FFT */
		gsl_fft_complex_radix2_inverse(packed,1,len);
	for(i=0;i<len;i++)
	{
		ReOut[i]=packed[i<<1];
		ImOut[i]=packed[(i<<1)+1];
	}
	memFree(packed);
}

#else	/* !HAVE_GSL */

#ifdef	USE_DUMB_FFT_IMPLEMENTATION

/*
 *	Nothing appropriate; fall back to our dumb implementation.
 *	Several times slower than NSP's fft()'s (accurate, although).
 */
void	V_GLOBAL
fftCalcNip(
	mvec_t *ReOut,mvec_t *ImOut,
	mvec_t *ReIn,mvec_t *ImIn,
	unsigned size_power,mvec_t dir_sign)
{
	mvec_t *xr, *xi, *yr, *yi, w, p, si, co, xa, xb, sc;
	unsigned size,m,k,l,i,j,ni,jm,k1,l1,p1,p2,pm,is,js;
	if(!ReOut || !ImOut || !ReIn || !ImIn || !size_power ||
		IS_POWER_TWO(size_power))
	{
#ifdef	CHECKED_BUILD
		conlPuts(LOG_ERROR,"fftCalcNip(): incorrect parameters.\n");
#endif	/* CHECKED_BUILD */
		return;
	}
	size=1+power2(size_power);
	xr=(mvec_t*)memAlloc(4*size*sizeof(mvec_t));
	xi=xr+size;
	yr=xi+size;
	yi=yr+size;
	size=(1+power2(size_power))*sizeof(mvec_t);
	memCopy(&xr[1],ReIn,size-sizeof(mvec_t));
	memCopy(&xi[1],ImIn,size-sizeof(mvec_t));
	p1=power2(size_power-1);
	for(m=1;m<=size_power;m++)
	{
		k1=power2(size_power-m)-1;
		pm=power2(m);
		p2=pm >> 1;
		p=M_PI/(mvec_t)p2;
		l1=p2-1;
		is=1;
		js=1;
		for(k=0;k<=k1;k++,is+=p2,js+=pm)
		{
			w=0;
			i=is;
			j=js;
			for(l=0;l<=l1;l++,i++,j++,w+=p)
			{
				ni=p1+i;
				jm=j+p2;
				v_fsincos(&si,&co,w);
				si*=dir_sign;
				xa=xr[ni]*co+xi[ni]*si;
				xb=xi[ni]*co-xr[ni]*si;
				yr[j]=xr[i]+xa;
				yi[j]=xi[i]+xb;
				yr[jm]=xr[i]-xa;
				yi[jm]=xi[i]-xb;
			}
		}
		memCopy(xr,yr,size);
		memCopy(xi,yi,size);
	}
	memCopy(ReOut,&xr[1],size-sizeof(mvec_t));
	memCopy(ImOut,&xi[1],size-sizeof(mvec_t));
	if(dir_sign < 0)
	{
		size=power2(size_power);
		sc=(mvec_t)1/(mvec_t)size;
		for(i=0;i<size;i++)
		{
			ReOut[i]*=sc;
			ImOut[i]*=sc;
		}
	}
	memFree(xr);
}

#else	/* !USE_DUMB_FFT_IMPLEMENTATION */

/*
 * Mainly, cut-and-paste from 'Numerical recipies in C'
 */
#define NRC_FFT_SWAP(a, b) tempr = (a); (a) = (b); (b) = tempr

V_LOCAL void nrc_four1(mvec_t data[], int nn, int isign)
/*
 * Replaces data by its discrete Fourier transform, if isign is input as 1;
 * or replaces data by nn times its inverse discrete Fourier transform,
 * if isign is input as -1.
 * data is a complex array of length nn, input as a real array data[1..2*nn].
 * nn MUST be an integer power of 2.
*/
{
	int i, n, mmax, m, j, istep;
	mvec_t wtemp, wr, wpr, wpi, wi, theta;
	mvec_t tempr, tempi;
	n = nn << 1;
	j = 1;
	/* This is the bit-reversal section of the routine. */
	for (i = 1; i < n; i += 2)
	{
    	if (j > i)
		{
			/* Exchange the two complex numbers. */
	  		NRC_FFT_SWAP(data[j  ],data[i  ]);
			NRC_FFT_SWAP(data[j+1],data[i+1]);
		};
		m = n >> 1;
		while (m >= 2 && j > m)
		{
			j  -= m;
			m >>= 1;
		};
		j += m;
	};
	/* Here begins the Danielson-Lanczos section of the routine. */
	mmax = 2;
	while (n > mmax)
	{
		/* Initialize trigonometric recurrence. */
		theta	= 6.28318530717959/(isign*mmax);
		wtemp	= sin(0.5*theta);
		wpr	= -2.0*wtemp*wtemp;
		wpi	= sin(theta);
		wr	= 1.0;
		wi	= 0.0;
		/* Here are the two nested inner loops. */
		istep	= 2*mmax;
		for (m = 1; m < mmax; m += 2)
		{
			for (i = m; i <= n; i += istep)
			{
				/* This is the Danielson-Lanczos formula. */
				j = i + mmax;
				tempr = wr*data[j  ] - wi*data[j+1];
				tempi = wr*data[j+1] + wi*data[j  ];
				data[j  ] = data[i  ] - tempr;
				data[j+1] = data[i+1] - tempi;
				data[i  ] += tempr;
				data[i+1] += tempi;
			}; /* Trigonometric recurrence. */
			wr = (wtemp = wr)*wpr - wi*wpi + wr;
			wi = wi*wpr + wtemp*wpi + wi;
		};
		mmax = istep;
	};
}

/*
 * Wrapper
 */
void	V_GLOBAL
fftCalcNip(
	mvec_t *ReOut,mvec_t *ImOut,
	mvec_t *ReIn,mvec_t *ImIn,
	unsigned size_power,mvec_t dir_sign)
{
	size_t i,len;
	mvec_t *packed;
	if(!ReOut || !ImOut || !ReIn || !ImIn || !size_power ||
		IS_POWER_TWO(size_power))
	{
#ifdef	CHECKED_BUILD
		conlPuts(LOG_ERROR,"fftCalcNip(): incorrect parameters.\n");
#endif	/* CHECKED_BUILD */
		return;
	}
	len=power2(size_power);
	packed=(mvec_t*)memAlloc((len << 1) * sizeof(mvec_t));
	for(i=0;i<len;i++)
	{
		packed[i<<1]=ReIn[i];
		packed[(i<<1)+1]=ImIn[i];
	}
	nrc_four1(packed,size_power,dir_sign > 0 ? 1 : -1);
	for(i=0;i<len;i++)
	{
		ReOut[i]=packed[i<<1];
		ImOut[i]=packed[(i<<1)+1];
	}
	memFree(packed);
}

#endif	/* !USE_DUMB_FFT_IMPLEMENTATION */

#endif	/* !HAS_NSP */

/*</pre></html>*/
/*EOF*/
