#ifndef	FSBASE_H
#define	FSBASE_H

#include <../source/TAL/objbase.h>
#include <../source/vfs/iobase.h>

/*
 *	Filesystem object
 */

typedef struct vfs_dev_s
{
	libv_object_t	base;
	u32_t	(V_CALL *get_inode_count)(struct vfs_dev_s *self);
	vboolean	(V_CALL *get_file_name)(struct vfs_dev_s *self,u32_t inode,char *name_ret);
	file_t	(V_CALL *create_file)(struct vfs_dev_s *self,u32_t inode);
}	vfs_dev_t, *vfs_dev_p;

/*
 *	FS tree leaf: represent file
 */
typedef struct	vfs_leaf_s
{
	char		name[_MAX_PATH];	/* Only name of file, without path */
	vfs_dev_t	*volume;			/* Filesystem of file */
	u32_t		inode;				/* Index of file in filesystem */
}	vfs_leaf_t, *vfs_leaf_p;

/*
 *	FS tree node (to store in <ntree_t*> -> data) - represent directory.
 */
typedef struct vfs_node_s
{
	char		name[_MAX_DIR];		/* Name of THIS node (relative name) */
	vp_stack_t	files;
	/* List of all filesystems: this fiels used by vfsOpenDir() */
	vp_stack_t	volumes;
}	vfs_node_t, *vfs_node_p;

/*
 *	Mode used by enumeration procedures.
 */
typedef enum
{
	vfs_mode_normal,
	vfs_mode_directory
}	vfs_mode_t;

/*
 *	Directory enumerator
 */
typedef struct vfs_enum_s
{
	libv_object_t base;
	/* private: */
	ntree_t		*node;
	char		node_path[_MAX_PATH];
	u32_t		num_nodes;
	u32_t		num_leafs;
	u32_t		node_num;
	u32_t		leaf_num;
	/* public: */
	vfs_dev_t	*vfs_dev;
	u32_t		vfs_ino;
	vfs_mode_t	vfs_mode;
	char		vfs_name[_MAX_PATH];
}	vfs_enum_t, *vfs_enum_p;

/*
 *	Mount procedure.
 *	Registered mount procedures will be called for every file
 *	by vfsInit().
 */
typedef vfs_dev_p	(V_CALL	*vfs_mount_proc_t)(char *volume_name);

#ifdef	__cplusplus
extern "C" {
#endif	/* __cplusplus */
	/* Creates filesystem class without any functionality */
	vfs_dev_p	V_CALL	vfsAlloc(u32_t size_of_object);
    /* Closes vfs handle */
    void	V_GLOBAL vfsDestroy(object_handle_t vfs_handle);
	/* Adds node to VFS tree. If 'path' is relative, it will be prepended
     * with current (local) working directory; "." and ".." names in 'path' 
	 * also handled properly. */
	vboolean	V_CALL	vfsAddNode(ntree_p root,char *path,
		vfs_dev_t *volume,u32_t inode);
	/*
	 *	Searches for node in VFS tree.
	 */
	vboolean	V_CALL	vfsGetNodeData(ntree_p root,char *full_path,vfs_dev_t **volume_ret,u32_t *inode_ret);
	/*
	 *	Uses vfsGetNodeData() to obtain device and inode of 'path_name',
	 *	than calls dev->create_file(inode). Returns NULL on error.
	 */
	file_t		V_CALL	vfsCreateFile(char *path_name);
	/*
	 *	Enumeration API's.
	 *	Normally, caller should use only 'vfs_name' field of
	 *	enumeration object.
	 */
	vfs_enum_p	V_CALL	vfsOpenDir(char *dir_name);
	vboolean	V_CALL	vfsReadDir(vfs_enum_p dir_enum);
	vboolean	V_CALL	vfsSeekDir(vfs_enum_p dir_enum,int offset,int whence);
	int			V_CALL	vfsTellDir(vfs_enum_p dir_enum);
	void		V_CALL	vfsCloseDir(vfs_enum_p dir_enum);
	void		V_CALL	vfsEnumFiles(char *vfs_path,
		void (V_CALL *dir_callback)(char *dir_name),
		void (V_CALL *file_callback)(char *file_name),
		vboolean recursive);
	/*
	 *	Adds filesystem factory to call from vfsInit()
	 */
	void		V_CALL	vfsRegisterMountProcedure(
		vfs_mount_proc_t mount_proc);
	/*
	 *		vfsInit() - searches for all local files starting from
	 *	current directory. Each file containing known file system mounted
	 *	at this local location; thus, vfs directory can contain files from
	 *	different file systems.
	 *		Currently, internally supported filesystems are:
	 *	- Id's PAK file system;
	 *	- EXE/DLL as resources (Win32 only, all compilers except WATCOM).
	 */
	void		V_CALL	vfsInit(void);
	/*
	 *	Empties VFS tree, unregisters all mount procedures.
	 */
	void		V_CALL	vfsShutdown(void);
#ifdef	__cplusplus
}
#endif	/* __cplusplus */

#endif	/* FSBASE_H */
